==> xdiagnose_3.2.tar.gz: xdiagnose/xdiagnose/edid.py <==
#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2010-2012 Bryce Harrington <bryce@canonical.com>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from __future__ import absolute_import, print_function, unicode_literals

import sys
import os
import re
import binascii

def loadfile(filename, binary=False):
    opentype = "r"
    if binary:
        opentype += "b"
    in_file = open(filename, opentype)
    text = in_file.read()
    in_file.close()
    return text

class Edid(object):
    def __init__(self, edid_raw=None):
        '''edid_raw: text bytecode such as from an Xorg.0.log'''
        self.edid_raw = None
        self._items = None
        self._origin = None
        if edid_raw:
            self.edid_raw = edid_raw
            self._origin = "custom"

    def load(self, filename):
        self.edid_raw = ""
        if filename is None or filename == '' or not os.path.exists(filename):
            return False

        try:
            # Try loading as a plain text file first (ala Xorg.0.log)
            text = loadfile(filename)
        except UnicodeDecodeError:
            # Now try again as binary
            data = loadfile(filename, binary=True)
            text = binascii.b2a_hex(data).decode("utf-8")

        if '00ffffffffffff00' == text[:16]:
            # Looks like a regular edid file
            self.edid_raw = text
        else:
            # Assume it's an Xorg.0.log
            re_head = re.compile("\(II\) .*\(\d+\): EDID \(in hex\):$")
            re_edid = re.compile("\(II\) .*\(\d+\):\s\t([0-9a-f]{32})$")

            seen_edid_header = False
            for line in text.split("\n"):
                if re_head.search(line):
                    seen_edid_header = True
                elif seen_edid_header:
                    m = re_edid.search(line)
                    if not m:
                        break
                    line = m.group(1)
                    self.edid_raw += line
        if self.edid_raw != "":
            self._origin = filename
            return True
        return False

    def _parse(self, edid):
        if edid is None:
            return None
        def _bytes(edid,byte1,byte2=None):
            if byte2:
                return edid[byte1*2:(byte2+1)*2]
            else:
                return edid[byte1*2:(byte1+1)*2]

        return [
            ("Header", _bytes(edid,0,7)),
            ("Manufacturer", _bytes(edid,8,9)),
            ("Product ID code", _bytes(edid,10,11)),
            ("Serial Number", _bytes(edid,12,15)),
            ("Week of Manufacture", _bytes(edid,16)),
            ("Year of Manufacture", _bytes(edid,17)),
            ("EDID Version", _bytes(edid,18)),
            ("EDID Revision", _bytes(edid,19)),

            ("Video input def", _bytes(edid,20)),
            ("Max Horiz Image(cm)", _bytes(edid,21)),
            ("Max Vert Image(cm)", _bytes(edid,22)),
            ("Gamma", _bytes(edid,23)),
            ("Power management", _bytes(edid,24)),

            ("Chromaticity", _bytes(edid,25,34)),
            ("Timing I", _bytes(edid,35)),
            ("Timing II", _bytes(edid,36)),
            ("Reserved Timing", _bytes(edid,37)),
            ("Standard Timing", _bytes(edid,38,53)),
            ("Horiz Active (px)", _bytes(edid,56)),
            ("Horiz Blanking", _bytes(edid,57)),
            ("Horiz high", _bytes(edid,58)),
            ("Vert Active", _bytes(edid,59)),
            ("Vert Blank", _bytes(edid,60)),
            ("Vert high", _bytes(edid,61)),
            ("Horz Sync Offset (px)", _bytes(edid,62)),
            ("Horiz Sync Pulse Width (px)", _bytes(edid,63)),
            ("Vert Sync (lines)", _bytes(edid,64)),
            ("high", _bytes(edid,65)),
            ("Horiz Image Size (mm)", _bytes(edid,66)),
            ("Vert Image Size (mm)", _bytes(edid,67)),
            ("Image Size high", _bytes(edid,68)),
            ("Horiz Border", _bytes(edid,69)),
            ("Vert Border", _bytes(edid,70)),
            ("Interlacing", _bytes(edid,71)),
            ("Descriptor Block 2", _bytes(edid,72,89)),
            ("Descriptor Block 3", _bytes(edid,90,107)),
            ("Descriptor Block 4", _bytes(edid,108,125)),
            ("Extension Flag", _bytes(edid,126)),
            ("Checksum", _bytes(edid,127)),
            ]

    @property
    def raw(self):
        return self.edid_raw

    @property
    def items(self):
        if self._items is None:
            self._items = self._parse(self.raw)
        return self._items

    @property
    def manufacturer(self):
        code = self.items[1][1]              # The ascii code
        bstr = str(bin(int(code, 16)))[2:]   # Converted to binary
        bstr = bstr.zfill(15)                # Fill left side with 0's
        s = [                                # Extract and convert letter codes to chars
            chr(int(bstr[-15:-10],2) + ord('A') - 1),
            chr(int(bstr[-10:-5],2) + ord('A') - 1),
            chr(int(bstr[-5:],2) + ord('A') - 1)
            ]
        return ''.join(s)

    @property
    def name(self):
        return "%s:%s  SN#%s  %d-%d  v%s.%s  %s" %(
            self.manufacturer,
            self.items[2][1],
            self.items[3][1],
            int(self.items[4][1], 16),
            int(self.items[5][1], 16) + 1990,
            self.items[6][1],
            self.items[7][1],
            self._origin
            )

    def __str__(self):
        text = ''
        for field, value in self.items:
            text += "%-30s: %s\n" %(field, value)
        return text


if __name__ == "__main__":
    edidfile = sys.argv[1]
    regex = re.compile("\(II\) .*\(\d+\):\s+(.*)$")

    edid_text = ""
    lines = loadtextfile(edidfile)
    for line in lines.split("\n"):
        m = regex.search(line)
        if m:
            line = m.group(1)
        edid_text += line

    edid = Edid(edid_text)
    print(edid)

==> xdiagnose_3.4.3.tar.gz: xdiagnose/xdiagnose/edid.py <==
#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2010-2012 Bryce Harrington <bryce@canonical.com>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from __future__ import absolute_import, print_function, unicode_literals

import sys
import os
import re
import binascii

from .utils.debug import (ERR, warn, dbg)
from .utils.file_io import (load_binary, load_file)


def _bytes(edid,byte1,byte2=None):
    if byte2:
        return edid[byte1*2:(byte2+1)*2]
    else:
        return edid[byte1*2:(byte1+1)*2]

# TODO: Split the firmware stuff into an EdidFirmware class
#       - _install(Edid)
#       - activate(Edid) - calls _install(Edid) automatically
#       - deactivate(Edid)

class Edid(object):
    EDID_FIRMWARE_PATH = '/lib/firmware/edid'
    EDID_DRM_CONF_PATH = '/etc/modprobe.d/drm-kms-helper.conf'

    def __init__(self, edid_raw=None):
        '''edid_raw: text bytecode such as from an Xorg.0.log'''
        self.edid_raw = None
        self._items = None
        self._origin = None
        if edid_raw:
            self.edid_raw = edid_raw
            self._origin = "custom"

    def install(self, edid_filename):
        '''Installs the named edid file into the firmware'''
        import errno
        import shutil
        try:
            os.makedirs(self.EDID_FIRMWARE_PATH)
        except OSError as exc:
            if exc.errno != errno.EEXIST:
                warn("Could not mkdir %s" %(self.EDID_FIRMWARE_PATH))
                return False

        # Install the EDID
        edid_firmware = os.path.basename(edid_filename)
        try:
            target = os.path.join(self.EDID_FIRMWARE_PATH, edid_firmware)
            shutil.copyfile(edid_filename, target)
            print("Installed %s" %(target))
        except:
            warn("Could not install firmware")
            raise
            return False
        return True

    def uninstall(self, filename):
        # TODO
        return True

    def activate(self, edid_name):
        '''Activates given edid by passing it as a kernel command line parameter'''
        f = open(self.EDID_DRM_CONF_PATH, 'w')
        f.write("options drm_kms_helper edid_firmware=edid/%s\n" %(edid_name))
        f.close()
        print("Activated %s via %s" %(edid_name, self.EDID_DRM_CONF_PATH))
        return True

    def save(self, filename):
        file = open(filename, 'wb')
        edid_binary = bytes.fromhex(self.edid_raw)
        file.write(edid_binary)
        file.close()
        return True

    def load(self, filename, edid_index=0):
        self.edid_raw = ""
        if filename is None or filename == '' or not os.path.exists(filename):
            return False

        try:
            # Try loading as a plain text file first (ala Xorg.0.log)
            lines = load_file(filename)
            if len(lines) < 1:
                return False

            if '00ffffffffffff00' == lines[0][:16]:
                # Looks like a regular edid file
                self.edid_raw = lines.join("\n")
                self._origin = filename
                return True

            # Assume it's an Xorg.0.log
            re_head = re.compile("\(II\) .*\(\d+\): EDID \(in hex\):$")
            re_edid = re.compile("\(II\) .*\(\d+\):\s\t([0-9a-f]{32})$")

            seen_edid_header = False
            header_index = 0
            for line in lines:
                if re_head.search(line):
                    seen_edid_header = True
                elif seen_edid_header:
                    m = re_edid.search(line)
                    if not m:
                        header_index += 1
                        continue
                    line = m.group(1)
                    if header_index == edid_index:
                        self.edid_raw += line
            if self.edid_raw != "":
                self._origin = filename
                return True

        except UnicodeDecodeError:
            # Now try again as binary
            data = load_binary(filename)
            self.edid_raw = binascii.b2a_hex(data).decode("utf-8")
            self._origin = filename
            return True

        return False

    def _parse(self, edid):
        if edid is None:
            return None
        return [
            ("Header", _bytes(edid,0,7)),
            ("Manufacturer", _bytes(edid,8,9)),
            ("Product ID code", _bytes(edid,10,11)),
            ("Serial Number", _bytes(edid,12,15)),
            ("Week of Manufacture", _bytes(edid,16)),
            ("Year of Manufacture", _bytes(edid,17)),
            ("EDID Version", _bytes(edid,18)),
            ("EDID Revision", _bytes(edid,19)),

            ("Video input def", _bytes(edid,20)),
            ("Max Horiz Image(cm)", _bytes(edid,21)),
            ("Max Vert Image(cm)", _bytes(edid,22)),
            ("Gamma", _bytes(edid,23)),
            ("Power management", _bytes(edid,24)),

            ("Chromaticity", _bytes(edid,25,34)),
            ("Timing I", _bytes(edid,35)),
            ("Timing II", _bytes(edid,36)),
            ("Reserved Timing", _bytes(edid,37)),
            ("Standard Timing", _bytes(edid,38,53)),
            ("Horiz Active (px)", _bytes(edid,56)),
            ("Horiz Blanking", _bytes(edid,57)),
            ("Horiz high", _bytes(edid,58)),
            ("Vert Active", _bytes(edid,59)),
            ("Vert Blank", _bytes(edid,60)),
            ("Vert high", _bytes(edid,61)),
            ("Horz Sync Offset (px)", _bytes(edid,62)),
            ("Horiz Sync Pulse Width (px)", _bytes(edid,63)),
            ("Vert Sync (lines)", _bytes(edid,64)),
            ("high", _bytes(edid,65)),
            ("Horiz Image Size (mm)", _bytes(edid,66)),
            ("Vert Image Size (mm)", _bytes(edid,67)),
            ("Image Size high", _bytes(edid,68)),
            ("Horiz Border", _bytes(edid,69)),
            ("Vert Border", _bytes(edid,70)),
            ("Interlacing", _bytes(edid,71)),
            ("Descriptor Block 2", _bytes(edid,72,89)),
            ("Descriptor Block 3", _bytes(edid,90,107)),
            ("Descriptor Block 4", _bytes(edid,108,125)),
            ("Extension Flag", _bytes(edid,126)),
            ("Checksum", _bytes(edid,127)),
            ]

    @property
    def raw(self):
        return self.edid_raw

    @property
    def items(self):
        if self._items is None:
            self._items = self._parse(self.raw)
        return self._items

    @property
    def manufacturer(self):
        code = self.items[1][1]              # The ascii code
        bstr = str(bin(int(code, 16)))[2:]   # Converted to binary
        bstr = bstr.zfill(15)                # Fill left side with 0's
        s = [                                # Extract and convert letter codes to chars
            chr(int(bstr[-15:-10],2) + ord('A') - 1),
            chr(int(bstr[-10:-5],2) + ord('A') - 1),
            chr(int(bstr[-5:],2) + ord('A') - 1)
            ]
        return ''.join(s)

    @property
    def product_id(self):
        return _bytes(self.raw,10,11)

    @property
    def serial_number(self):
        return _bytes(self.raw,12,15)

    @property
    def name(self):
        return "%s:%s  SN#%s  %d-%d  v%s.%s  %s" %(
            self.manufacturer,
            self.items[2][1],
            self.items[3][1],
            int(self.items[4][1], 16),
            int(self.items[5][1], 16) + 1990,
            self.items[6][1],
            self.items[7][1],
            self._origin
            )

    def __str__(self):
        text = ''
        for field, value in self.items:
            text += "%-30s: %s\n" %(field, value)
        return text


if __name__ == "__main__":
    edidfile = sys.argv[1]
    regex = re.compile("\(II\) .*\(\d+\):\s+(.*)$")

    edid_text = ""
    lines = load_file(edidfile)
    for line in lines.split("\n"):
        m = regex.search(line)
        if m:
            line = m.group(1)
        edid_text += line

    edid = Edid(edid_text)
    print(edid)
